/********************************************************************
 	created:  10/10/2006  
	filename: KernelObject.h
 	
	purpose:	
*********************************************************************/
#ifndef _KernelObject_h_10_10_2006
#define _KernelObject_h_10_10_2006



// check handle for NULL and INVALID_HANDLE
inline BOOL IsValidHandle( HANDLE hHandle) 
{
    return ((hHandle != NULL) && (hHandle != INVALID_HANDLE_VALUE));
}

// validate wait return codes...
inline BOOL WaitSucceeded( DWORD dwWaitResult, DWORD dwHandleCount) 
{
    return ((dwWaitResult >= WAIT_OBJECT_0) &&
            (dwWaitResult < WAIT_OBJECT_0 + dwHandleCount));
}

inline BOOL WaitAbandoned( DWORD dwWaitResult, DWORD dwHandleCount) 
{
    return ((dwWaitResult >= WAIT_ABANDONED_0) &&
            (dwWaitResult < WAIT_ABANDONED_0 + dwHandleCount));
}

inline BOOL WaitTimeout( DWORD dwWaitResult) 
{
    return (dwWaitResult == WAIT_TIMEOUT);
}
    
inline BOOL WaitFailed( DWORD dwWaitResult) 
{
    return (dwWaitResult == WAIT_FAILED);
}

// compute object indices for waits...
inline DWORD WaitSucceededIndex( DWORD dwWaitResult) 
{
    return (dwWaitResult - WAIT_OBJECT_0);
}

inline DWORD WaitAbandonedIndex( DWORD dwWaitResult) 
{
    return (dwWaitResult - WAIT_ABANDONED_0);
}

class __declspec(dllexport) CKernelObject
{
public:
    // destructor is virtual to make CKernelObject an abstract base class...
    virtual ~CKernelObject() = 0;

    // read the creation status of the internal kernel object...
    DWORD Status(void) const;

    // wait on the current kernel object...
    DWORD Wait( DWORD dwMilliseconds);

    // wait on the current object and one other...
    DWORD WaitForTwo( CKernelObject &rWaitableObject, BOOL bWaitAll, DWORD dwMilliseconds);

    // get the internal handle...
    HANDLE GetHandle(void) const;

    // another way to get the internal handle...
    operator HANDLE() const;

private:
    // these functions have no implementation since they can 
    // never be called...

    // copying and passing by copy are not allowed...
    // this prevents confusion of internal object ownership...
    CKernelObject(CKernelObject & rhs);

    // assigning one object to another is not allowed,
    // this prevents confusion of internal object ownership...
    CKernelObject & operator= (CKernelObject & rhs);

protected:
    HANDLE m_hHandle;
    DWORD m_dwStatus;

protected:
    // constructor...
    CKernelObject();

    // error handling...
    void ThrowError( DWORD dwStatus, string const& s) ;

};

#endif
