/********************************************************************
 	created:  10/10/2006  
	filename: ShmQueue.cpp
 	
	purpose:	
*********************************************************************/

//
// SYSTEM INCLUDE
//

//
// PROJECT INCLUDE
//
#include "StdAfx.h"

//
// LOCAL INCLUDE
//
#include "KernelObject.h"
#include "semaphore.h"
#include "shm.h"
#include "mutex.h"
#include "Event.h"

 #include "ShmQueue.h"

struct AutoLockS
{
	AutoLockS(CMutex*p):m_pLock(p)
	{
		m_pLock->Wait(INFINITE);
	}

	~AutoLockS()
	{
		m_pLock->Release();
	}

	CMutex * m_pLock;
};

void CShmQueue::InitSharedMemoryContent(int shmsize)
{
 	
	NOTIFYBUFFER& loop_buffer=m_pSharedBuffer->notify_buffer;
	loop_buffer.TotalSize=shmsize-sizeof(SHARED_BUFFER);
	loop_buffer.WIndex=0;
	loop_buffer.RIndex=0;
	loop_buffer.UnReadDataAvail=0;
	m_pSharedBuffer->ulSig=0xbeefdead;

}

CShmQueue::CShmQueue(string const&strShmid, 
				string const& semaphoreId, 
				int shmsize,
				int flag): 
m_sem(0, 100,(LPCTSTR)semaphoreId.c_str()),
m_shm(shmsize,(LPCTSTR)strShmid.c_str())
{
	m_pSharedBuffer=(PSHARED_BUFFER)m_shm.GetPtr();
	InitSharedMemoryContent(shmsize);
}

				
CShmQueue::CShmQueue(	string const&strShmid, 
					 string const& semaphoreId)  :

m_sem((LPCTSTR)semaphoreId.c_str()),
m_shm((LPCTSTR)strShmid.c_str())
{
	m_pSharedBuffer=(PSHARED_BUFFER)m_shm.GetPtr();
}


bool CShmQueue::Write(char* pdata, size_t writeSize)
{
 
	//
	// If loop buffer is full, return false
	//
	NOTIFYBUFFER& ring_buffer=m_pSharedBuffer->notify_buffer;

	long dataFieldSize=(ULONG)(writeSize+ 3)& ~3;
 	long totalDataSize =  sizeof(unsigned long)+dataFieldSize;

 
	// Get a big enough slot in the ring buffer
	if(InterlockedExchangeAdd(&ring_buffer.UnReadDataAvail, totalDataSize) + totalDataSize > ring_buffer.TotalSize)
	{
		InterlockedExchangeAdd(&ring_buffer.UnReadDataAvail, -totalDataSize);
		return false;
	}

	LONG write_location = InterlockedExchangeAdd(&ring_buffer.WIndex, totalDataSize);
	write_location %= ring_buffer.TotalSize;

	// Note where we are going to write the size field - NB we do this last to stop the reader
	// reading before we're ready!
	DWORD* size_field = (DWORD*) (ring_buffer.buffer + write_location);
	write_location = (write_location + sizeof(DWORD)) % ring_buffer.TotalSize;

	if(write_location + dataFieldSize > ring_buffer.TotalSize)
	{
		// Split in two
		unsigned long first_chunk_size = ring_buffer.TotalSize - write_location;

		memcpy(ring_buffer.buffer + write_location,pdata,first_chunk_size);
		memcpy(ring_buffer.buffer,pdata+first_chunk_size,dataFieldSize-first_chunk_size);
 	}
	else
	{
		memcpy(ring_buffer.buffer + write_location,pdata,dataFieldSize);
 	}

	*size_field = totalDataSize;

	m_sem.Release(1);
	return true;
}

ReadStatus CShmQueue::Read(vector<char>&entry)
{
	DWORD res=m_sem.WaitForTwo(m_releaseEvent,FALSE,INFINITE);

	if (!WaitSucceeded(res,2))
		return PROGRAM_INTERNAL_ERROR;

	//
	// Check if it is a quit command
	//
	if (WaitSucceededIndex(res)==1)
	{
		return RELEASED;
	}

	//
	// The semaphore is signaled, process the event
	//

	//
	// The lock is only for multiple read
	//
	// Note: We don't have lock in write access because the code was carefully
	// design to avoid conflict between write and read. However, we have conflict
	// between reads and we set lock here.
	//

	
	NOTIFYBUFFER& ring_buffer=m_pSharedBuffer->notify_buffer;
	if(InterlockedCompareExchange(&ring_buffer.UnReadDataAvail, 0, 0) == 0) 
	return PROGRAM_INTERNAL_ERROR;

	// Buffer format is DWORD length, followed by data
	LONG entry_size = *(LONG*)(ring_buffer.buffer + ring_buffer.RIndex);
	LONG realDataSize =entry_size-sizeof(unsigned long);
//	ASSERT(entry_size < ring_buffer.TotalSize);

	// If entry not yet ready, the size won't have been set
	if(!entry_size) 
	{
		entry.resize(0);
		return SUCCESS_WITHOUT_DATA;
	}
	
	//
	// Clean up this entry
	//
	// Note: It does not matter 
	*(DWORD*)(ring_buffer.buffer + ring_buffer.RIndex) = 0;

	// Copy the  data to a vector
 
	entry.resize(realDataSize); 
	BYTE* start =  ring_buffer.buffer + ring_buffer.RIndex+sizeof(unsigned long);
	BYTE* end =   ring_buffer.buffer + ring_buffer.RIndex + entry_size;
	BYTE* BufferEnd =  ring_buffer.buffer + ring_buffer.TotalSize ;

	if(end > BufferEnd)
	{
		int remaining = ring_buffer.TotalSize - ring_buffer.RIndex;

		copy( start, BufferEnd, entry.begin());
 		copy(ring_buffer.buffer, ring_buffer.buffer + realDataSize - remaining, entry.begin() + remaining);
	}
	else
	{
		copy(start, end, entry.begin());
	}

	// Allow this space to be reused.
	ring_buffer.RIndex = (ring_buffer.RIndex + entry_size) % ring_buffer.TotalSize;
	InterlockedExchangeAdd(&ring_buffer.UnReadDataAvail, -entry_size);
	return SUCCESS_WITHDATA;
}

void CShmQueue::Release()
{
	m_releaseEvent.Set();
}