/********************************************************************
 	created:  10/10/2006  
	filename: shm.cpp
 	
	purpose:	
*********************************************************************/

//
// SYSTEM INCLUDE
//
#include "stdafx.h"
//
// PROJECT INCLUDE
//

//
// LOCAL INCLUDE
//
#include "shmqueueexception.h"
#include "shm.h"

CSharedMemory::CSharedMemory( LONG lSize, 
							 LPCTSTR lpName, 
                             LPSECURITY_ATTRIBUTES lpSharedMemoryAttributes) : 
m_lpSharedMemory(NULL),
m_bIsCreator(FALSE),
m_dwStatus(NO_ERROR) 
{
    m_hMapping = ::CreateFileMapping( (HANDLE) -1, lpSharedMemoryAttributes, PAGE_READWRITE, 0, lSize, lpName);
    if (IsValidHandle(m_hMapping)) 
	{
        m_dwStatus = GetLastError();
        if (m_dwStatus == ERROR_ALREADY_EXISTS) 
		{
            m_bIsCreator = FALSE;
        }
        else 
		{
            m_bIsCreator = TRUE;
        }
    }
    else 
	{
        m_dwStatus = ::GetLastError();
		throw CShmQueueException(m_dwStatus,string("Unable to create memory mapping file"));
	}

    m_lpSharedMemory = (void *) ::MapViewOfFile( m_hMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    
    if (m_lpSharedMemory == NULL) 
	{
        // unable to map memory into address space, perhaps our address space is full?
        // this is a fatal error, close handle and throw exeption
        m_dwStatus = ::GetLastError();
 
		throw CShmQueueException(m_dwStatus,string("Unable to map memory into user space"));
    }

}

CSharedMemory::CSharedMemory( LPCTSTR lpName) : 
m_lpSharedMemory(NULL),
m_bIsCreator(FALSE),
m_dwStatus(NO_ERROR) 
{
    m_hMapping = ::OpenFileMapping( FILE_MAP_ALL_ACCESS, FALSE, lpName);
    if (IsValidHandle(m_hMapping)) 
	{
        m_dwStatus = NO_ERROR;
    }
    else 
	{
        m_dwStatus = ::GetLastError();
		throw CShmQueueException(m_dwStatus,string("Unable to open shared memory"));
    }

    m_lpSharedMemory = (void *) ::MapViewOfFile( m_hMapping, FILE_MAP_ALL_ACCESS, 0, 0, 0);
    
    if (m_lpSharedMemory == NULL) 
	{
        m_dwStatus = ::GetLastError();
		throw CShmQueueException(m_dwStatus,string("Unable to map shared memory into user space"));
    }
}
    
CSharedMemory::~CSharedMemory() 
{
    if (m_lpSharedMemory) 
	{
        ::UnmapViewOfFile(m_lpSharedMemory);
        m_lpSharedMemory = NULL;
    }

    // close the file mapping handle...
    if (m_hMapping) 
	{
        ::CloseHandle(m_hMapping);
    }
}

DWORD CSharedMemory::Status(void) const 
{
    return m_dwStatus;
}

void * CSharedMemory::GetPtr(void) 
{
    return m_lpSharedMemory;
}

BOOL CSharedMemory::IsCreator(void) 
{
    return m_bIsCreator;
}

